import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import java.util.Vector;

public class Cryptex extends MIDlet
{ 
    static final Command QUIT_CMD = new Command(Locale.QUIT, Command.BACK, 3);
    static final Command BACK_CMD = new Command(Locale.BACK, Command.BACK, 2);
    static final Command ENTER_CMD = new Command(Locale.ENTER, Command.OK, 1);
    static final Command UPDATE_CMD = new Command(Locale.UPDATE, Command.OK, 1);
    static final Command ADD_CMD = new Command(Locale.ADD, Command.OK, 1);
    static final Command SELECT_CMD = new Command(Locale.SELECT, Command.ITEM, 1);
    static final Command CHANGE_PASSWORD_CMD = new Command(Locale.CHANGE_PASSWORD, Command.SCREEN, 3);
    static final Command REMOVE_CMD = new Command(Locale.REMOVE, Command.ITEM, 2);

    RecordStore store;
    Vector      content;
    Rc4         coder;
    Record      question;

    static final String STORE_NAME = "CRYPTEX";
    static final String ONLY_DIGITS = "1234";
    static final String ANY_CHARACTER = "abcd";


    public Cryptex() { 
        content = new Vector();
    }

    protected void startApp() 
    {
        try { 
            try { 
                store = RecordStore.openRecordStore(STORE_NAME, false);
                try { 
                    byte[] record = store.getRecord(1);
                    question = new Record(1, record);
                    new EnterPasswordForm(this, question);
                } catch (InvalidRecordIDException x) {
                    new SetPasswordForm(this, null);
                }
            } catch (RecordStoreNotFoundException x) {             
                store = RecordStore.openRecordStore(STORE_NAME, true);
                new SetPasswordForm(this, null);
            }             
        } catch(Exception x) { 
            criticalError(x);
        }
    }

    void loadData() { 
        try { 
            int last = store.getNextRecordID();
            for (int rid = 2; rid < last; rid++) { 
                byte[] record = null;
                try { 
                    record = store.getRecord(rid);                    
                } catch (InvalidRecordIDException x) {
                    // record was deleted 
                }
                if (record != null) { 
                    content.addElement(new Record(rid, coder.decrypt(record)));
                }
            }
        } catch(Exception x) { 
            criticalError(x);
        }
    }
    
    void addRecord(String key, String value) { 
        Record rec = new Record(key, value);
        content.addElement(rec);
        byte[] record = rec.pack();
        record = coder.encrypt(record);
        try { 
            rec.rid = store.addRecord(record, 0, record.length);
        } catch (Exception x) {
            criticalError(x);
        }
    }
    
    void updateRecord(int pos, String key, String value) { 
        Record rec = (Record)content.elementAt(pos);
        rec.key = key;
        rec.value = value;
        byte[] record = rec.pack();
        record = coder.encrypt(record);
        try { 
            store.setRecord(rec.rid, record, 0, record.length);
        } catch (Exception x) {
            criticalError(x);
        }
    }

    void deleteRecord(int pos) { 
        Record rec = (Record)content.elementAt(pos);
        content.removeElementAt(pos);
        try { 
            store.deleteRecord(rec.rid);                        
        } catch (Exception x) {
            criticalError(x);
        }
    }
         
    void setPassword(String passwordQuestion, String password) { 
        boolean onlyDigits = true;
        for (int i = password.length(); --i >= 0;) { 
            if (!Character.isDigit(password.charAt(i))) { 
                onlyDigits = false;
                break;
            }
        }
        question = new Record(passwordQuestion, onlyDigits ? ONLY_DIGITS : ANY_CHARACTER);
        byte[] record = question.pack();
        try { 
            store.addRecord(record, 0, record.length);
        } catch (Exception x) {
            criticalError(x);
        }
        coder = new Rc4(password);
    }

    void changePassword(String passwordQuestion, String password) { 
        boolean onlyDigits = true;
        for (int i = password.length(); --i >= 0;) { 
            if (!Character.isDigit(password.charAt(i))) { 
                onlyDigits = false;
                break;
            }
        }
        question = new Record(passwordQuestion, onlyDigits ? ONLY_DIGITS : ANY_CHARACTER);
        coder = new Rc4(password);
        try { 
            int n = content.size();
            int[] rid = new int[n];
            for (int i = 0; i < n; i++) { 
                Record rec = (Record)content.elementAt(i);
                byte[] record = rec.pack();
                rid[i] = rec.rid;
                record = coder.encrypt(record);
                rec.rid = store.addRecord(record, 0, record.length);
            } 
            byte[] record = question.pack();
            store.setRecord(1, record, 0, record.length);
            for (int i = 0; i < n; i++) { 
                store.deleteRecord(rid[i]);
            }
        } catch (Exception x) {
            criticalError(x);
        }    
    }

    void error(String text) { 
        Alert alert = new Alert(Locale.ERROR, text, null, AlertType.ERROR);
        alert.setTimeout(Alert.FOREVER);
        Display.getDisplay(this).setCurrent(alert);     
    }

    void criticalError(Exception x) { 
        error(x.toString());
    }
    

    protected void destroyApp(boolean unconditional) 
    {
        try { 
            store.closeRecordStore();
        } catch (Exception x) {
            criticalError(x);
        }
    }  

    protected  void pauseApp() 
    {
    }

    void quit() { 
        destroyApp(true);
        notifyDestroyed();
    }
}
